package java.beans;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.EventListener;
import java.util.Hashtable;
import java.util.Map;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamField;
import java.io.Serializable;

public class PropertyChangeSupport implements Serializable
{
    private PropertyChangeSupport.PropertyChangeListenerMap map;
    private Object source;
    private static final ObjectStreamField[] serialPersistentFields;
    static final long serialVersionUID = 6401253773779951803L;
    
    public PropertyChangeSupport(Object source) {
        this.map = new PropertyChangeSupport.PropertyChangeListenerMap();
        if (source == null) {
            throw new NullPointerException();
        }
        this.source = source;
        return;
    }
    
    public void addPropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        if (propertyChangeListener == null) {
            return;
        }
        if ((propertyChangeListener instanceof PropertyChangeListenerProxy)) {
            PropertyChangeListenerProxy propertyChangeListenerProxy = (PropertyChangeListenerProxy)propertyChangeListener;
            this.addPropertyChangeListener(propertyChangeListenerProxy.getPropertyName(), (PropertyChangeListener)propertyChangeListenerProxy.getListener());
        }
        else {
            this.map.add(null, propertyChangeListener);
        }
        return;
    }
    
    public void removePropertyChangeListener(PropertyChangeListener propertyChangeListener) {
        if (propertyChangeListener == null) {
            return;
        }
        if ((propertyChangeListener instanceof PropertyChangeListenerProxy)) {
            PropertyChangeListenerProxy propertyChangeListenerProxy = (PropertyChangeListenerProxy)propertyChangeListener;
            this.removePropertyChangeListener(propertyChangeListenerProxy.getPropertyName(), (PropertyChangeListener)propertyChangeListenerProxy.getListener());
        }
        else {
            this.map.remove(null, propertyChangeListener);
        }
        return;
    }
    
    public PropertyChangeListener[] getPropertyChangeListeners() {
        return (PropertyChangeListener[])this.map.getListeners();
    }
    
    public void addPropertyChangeListener(String s, PropertyChangeListener extract) {
        if (extract == null || s == null) {
            return;
        }
        extract = this.map.extract(extract);
        if (extract != null) {
            this.map.add(s, extract);
        }
        return;
    }
    
    public void removePropertyChangeListener(String s, PropertyChangeListener extract) {
        if (extract == null || s == null) {
            return;
        }
        extract = this.map.extract(extract);
        if (extract != null) {
            this.map.remove(s, extract);
        }
        return;
    }
    
    public PropertyChangeListener[] getPropertyChangeListeners(String s) {
        return (PropertyChangeListener[])this.map.getListeners(s);
    }
    
    public void firePropertyChange(String s, Object o, Object obj) {
        if (o == null || obj == null || !o.equals(obj)) {
            this.firePropertyChange(new PropertyChangeEvent(this.source, s, o, obj));
        }
    }
    
    public void firePropertyChange(String s, int i, int j) {
        if (i != j) {
            this.firePropertyChange(s, Integer.valueOf(i), Integer.valueOf(j));
        }
    }
    
    public void firePropertyChange(String s, boolean b, boolean b2) {
        if (b != b2) {
            this.firePropertyChange(s, Boolean.valueOf(b), Boolean.valueOf(b2));
        }
    }
    
    public void firePropertyChange(PropertyChangeEvent propertyChangeEvent) {
        Object oldValue = propertyChangeEvent.getOldValue();
        Object newValue = propertyChangeEvent.getNewValue();
        if (oldValue == null || newValue == null || !oldValue.equals(newValue)) {
            String propertyName = propertyChangeEvent.getPropertyName();
            PropertyChangeListener[] array = (PropertyChangeListener[])this.map.get(null);
            PropertyChangeListener[] array2 = (propertyName != null) ? ((PropertyChangeListener[])this.map.get(propertyName)) : null;
            fire(array, propertyChangeEvent);
            fire(array2, propertyChangeEvent);
        }
    }
    
    private static void fire(PropertyChangeListener[] array, PropertyChangeEvent propertyChangeEvent) {
        if (array != null) {
            for (int length = array.length, i = 0; i < length; ++i) {
                array[i].propertyChange(propertyChangeEvent);
            }
        }
    }
    
    public void fireIndexedPropertyChange(String s, int n, Object o, Object obj) {
        if (o == null || obj == null || !o.equals(obj)) {
            this.firePropertyChange(new IndexedPropertyChangeEvent(this.source, s, o, obj, n));
        }
    }
    
    public void fireIndexedPropertyChange(String s, int n, int i, int j) {
        if (i != j) {
            this.fireIndexedPropertyChange(s, n, Integer.valueOf(i), Integer.valueOf(j));
        }
    }
    
    public void fireIndexedPropertyChange(String s, int n, boolean b, boolean b2) {
        if (b != b2) {
            this.fireIndexedPropertyChange(s, n, Boolean.valueOf(b), Boolean.valueOf(b2));
        }
    }
    
    public boolean hasListeners(String s) {
        return this.map.hasListeners(s);
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Object o = null;
        PropertyChangeListener[] array = null;
        synchronized (this.map) {
            for (Map.Entry entry : this.map.getEntries()) {
                String key = (String)entry.getKey();
                if (key == null) {
                    array = (PropertyChangeListener[])entry.getValue();
                }
                else {
                    if (o == null) {
                        o = new Hashtable();
                    }
                    PropertyChangeSupport value = new PropertyChangeSupport(this.source);
                    value.map.set(null, (EventListener[])entry.getValue());
                    o.put(key, value);
                }
            }
        }
        ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
        putFields.put("children", o);
        putFields.put("source", this.source);
        putFields.put("propertyChangeSupportSerializedDataVersion", 2);
        objectOutputStream.writeFields();
        if (array != null) {
            for (PropertyChangeListener obj : array) {
                if ((obj instanceof Serializable)) {
                    objectOutputStream.writeObject(obj);
                }
            }
        }
        objectOutputStream.writeObject(null);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
        this.map = new PropertyChangeSupport.PropertyChangeListenerMap();
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        Hashtable<Object, Object> hashtable = (Hashtable)fields.get("children", null);
        this.source = fields.get("source", null);
        fields.get("propertyChangeSupportSerializedDataVersion", 2);
        Object object;
        while (null != (object = objectInputStream.readObject())) {
            this.map.add(null, (PropertyChangeListener)object);
        }
        if (hashtable != null) {
            for (Map.Entry entry : hashtable.entrySet()) {
                PropertyChangeListener[] propertyChangeListeners = ((PropertyChangeSupport)entry.getValue()).getPropertyChangeListeners();
                for (int length = propertyChangeListeners.length, i = 0; i < length; ++i) {
                    this.map.add((String)entry.getKey(), propertyChangeListeners[i]);
                }
            }
        }
    }
    
    static {
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("children", Hashtable.class), new ObjectStreamField("source", Object.class), new ObjectStreamField("propertyChangeSupportSerializedDataVersion", Integer.TYPE) };
    }
    
    private static final class PropertyChangeListenerMap extends ChangeListenerMap<PropertyChangeListener>
    {
        private static final PropertyChangeListener[] EMPTY;
        
        @Override
        protected PropertyChangeListener[] newArray(int n) {
            return (0 < n) ? new PropertyChangeListener[n] : PropertyChangeSupport.PropertyChangeListenerMap.EMPTY;
        }
        
        @Override
        protected PropertyChangeListener newProxy(String s, PropertyChangeListener propertyChangeListener) {
            return new PropertyChangeListenerProxy(s, propertyChangeListener);
        }
        
        @Override
        public final PropertyChangeListener extract(PropertyChangeListener propertyChangeListener) {
            while (propertyChangeListener instanceof PropertyChangeListenerProxy) {
                propertyChangeListener = (PropertyChangeListener)((PropertyChangeListenerProxy)propertyChangeListener).getListener();
            }
            return propertyChangeListener;
        }
        
        static {
            EMPTY = new PropertyChangeListener[0];
        }
    }
}
